#!/bin/sh

# binfmt_misc
if [ ! -d /proc/sys/fs/binfmt_misc ]; then
	echo "Host kernel is lacking binfmt_misc support!"
	echo "You need it to run scratchbox."
	exit 1
fi
					

# VDSO
if [ -f "/proc/sys/kernel/vdso" ] ; then
	if [ `cat /proc/sys/kernel/vdso` == 1 ] ; then
		echo "Host kernel has vdso support (which is uncompatible with SB)"
		echo "You can fix this with either: "
		echo "  echo 0 > /proc/sys/kernel/vdso"
		echo "or"
		echo "  add 'vdso=0' to the kernel parameters"
		exit 1
	fi
fi

# ip_local_port_range

if cat /proc/sys/net/ipv4/ip_local_port_range | awk '{ exit ! ($2-$1 < 10000) }'; then 
    echo "Host kernel has port range under 10000."
    echo "This causes problems with fakeroot."
    echo "You can fix this eg. by running:"
    echo '  echo "1024 65000" > /proc/sys/net/ipv4/ip_local_port_range'    
    exit 1
fi