/*
 * Copyright (C) 2006, Movial Corporation
 * Author: Toni Timonen <ttimonen@movial.fi>
 *
 * Contains some parts made by
 * Lauri Leukkunen <lauri.leukkunen@nokia.com>
 * Copyright (C) 2004, 2005 Nokia
 *
 *
 * Licenced under GPL, see COPYING for details.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define LIB_DIR        "/scratchbox/host_shared/lib/"
#define LD_PATH        LIB_DIR "ld.so"

static int endswith(char *haystack,char *needle) {
	return strcmp(&haystack[strlen(haystack)-strlen(needle)],needle)==0;
}

static int oneStartswith(char **data,char *key) {
	int len=strlen(key);
	if(!data) return 0;
	char **p;
	for(p=data;*p;p++) if (strncmp(*p,key,len)==0) return 1;
	return 0;
}

static int elemCount(char **elems) {
	int count=0;
	char **p=elems;
	while(*p) {p++; count++;}
	return count;
}

// Returns the number of splittable elements in s
static int substrCount(char *s,char sep) {
	int count=1;
	char *p;
	if(!s)
		return 0;
	for(p=s;*p;p++) if(*p==sep) count++;
	return count;
}

static int isLinker(char *fname) {
	return endswith(fname,"ld");
}

static char**paddedList(char **list) {
	if(!list) {
		list=(char**)calloc(1,sizeof(char*));
	}
	return list;
}

#define ENVLEN (2)
char *envs[]={"LD_LIBRARY_PATH","LD_RUN_PATH"};
#define LINK_FLAG "-rpath-link"

char **pre_args(char *fname,char **args) {
	char **ret=NULL;
	if (isLinker(fname) && !oneStartswith(args,"-rpath")) {
		int nOfElems=0;
		int i=0,j=0;
		// Figure out list size.
		for(;i<ENVLEN;i++)
				nOfElems+=(getenv(envs[i])!=NULL);
		ret=(char**)calloc(nOfElems*2+1,sizeof(char*));
		ret[nOfElems]=NULL;

		// Fill the list
		// 
		for(i=0,j=0;i<ENVLEN;i++) {
			char *p=getenv(envs[i]);
			if(!p) continue;
			ret[j++]=strdup(LINK_FLAG);
			ret[j++]=strdup(p);
		}

	}
	return paddedList(ret);
}

char *searchdir_flags[]={"-L/usr/local/lib","-L/lib","-L/usr/lib",NULL};
#define SD_FLAGLEN (3)

char **post_args(char *fname,char **args) {
	char **ret=NULL;
	if(isLinker(fname)) 
		ret=searchdir_flags;
	return paddedList(ret);
	
}

int main(int argc, char **argv) {
	char *binaryname, *exe, **my_argv;
	char **pre,**post;
	char **p;
	int i = 0, j;

	binaryname = basename(strdup(argv[1]));


	exe = argv[1];

	pre=pre_args(binaryname,&argv[2]);
	post=post_args(binaryname,&argv[2]);

	my_argv = (char**)calloc(4 + (argc - 1) + 1+elemCount(pre)+elemCount(post), sizeof (char *));
	my_argv[i++] = LD_PATH;
	my_argv[i++] = "--library-path";
	my_argv[i++] = LIB_DIR;
	my_argv[i++] = exe;

	for(p=pre;*p;p++) my_argv[i++]=*p;

	for (j = 2; j < argc;  ++j)
		my_argv[i++] = argv[j];

	for(p=post;*p;p++) my_argv[i++]=*p;


	execv(my_argv[0], my_argv);

	fprintf(stderr, "sb_alien (running %s): %s\n", exe, strerror(errno));
	return -11;
}

